@@ -1,11 +1,12 @@
 # !/usr/bin/env python
 # -*- coding:utf-8 -*-
 
-# (c)Ing. Zdenek Dlauhy, Michal Dlauhý, pripravto.cz
+# (c)Ing. Zdenek Dlauhy, Michal Dlauhý, info@pripravto.cz, www.pripravto.cz
 
 
 import subprocess
 import os
+import functools
 
 # TODO: fix logging and messages
 # import logging
@@ -27,6 +28,7 @@ class log(object):
 try:
     import ansible.runner
     import ansible.inventory
+    from ansible.utils.plugins import module_finder
 except ImportError as e:
     log.error("Cannot import Ansible! Ansiblator will not work. {}".format(e))
     log.error("Run:  pip install ansible")
@@ -40,19 +42,17 @@ Classes
 
 Ansiblator() - actuall wrapper
 DictToInventory - convertor for inventory files
-
-
 """
 try:
     from ansiblator.version import __version__
 except ImportError:
     __version__ = 0
 
-__version__ = "0.5-{}".format(__version__)
+__version__ = "0.6-{}".format(__version__)
 __author__ = "pripravto-service"
 __email__ = "test@dlauhy.cz"
 __url__ = "http://www.pripravto.cz"
-__desc__ = "Ansiblator - makes Ansible api more pythonic"
+__desc__ = "Ansiblator - makes Ansible api more Pythonic"
 __sys_name__ = "ansiblator"
 __license__ = "GPLv3"
 
@@ -83,6 +83,7 @@ class DictToInventory(object):
         self.filename = filename
         self.dict_to_inverntory(dict_data)
         self.data_to_text()
+        log.info("Dictionary converted to ansible host file {}".format(filename))
 
     def dict_to_inverntory(self, dict_data):
         """convert dict data into inventory file"""
@@ -124,6 +125,8 @@ class Ansiblator(object):
     Ansiblator
     ==========
 
+    Ansiblator - makes Ansible api more Pythonic
+
     This wrapper allows more easier way how to use Ansible in Python.
     Chain commands without without playbooks. More like Fabric.
 
@@ -189,15 +192,23 @@ class Ansiblator(object):
                 self.run("apt-get update", su=su, sudo=sudo)
                 self.run("apt-get upgrade -y", su=su, sudo=sudo)
 
-    use class and more patterns::
+    use custom class and more patterns together::
 
         ans = Automatization(pattern=['servers', 'production', 'test', 'pc'])
         ans.update_server()
 
-
     With this, you can create full commands or functions and just pass to them
     pattern and run at the end.
 
+    Need all modules inside Ansible?::
+
+        ans = an.Ansiblator()
+        ans.get_all_modules()
+        #now you should be able to do >
+        ans.user(name="hugo")
+        #or even
+        ans.pip(name="six", virtualenv="/tmp/venv", virtualenv_site_packages="yes")
+
 
     More information
     ================
@@ -215,22 +226,23 @@ class Ansiblator(object):
         return_code = ans.local(["uname", "-a"], now=True, use_shell=False)
         return_code['contacted']
 
-    TODO
+    Todo
     ====
 
     - make more tests
-    - make automatic translating of methods to ansible modules
     - improve logging
     - improve DictToInventory mapper, so more options are possible, such as groups and so on
 
-    CHANGES
+    Changes
     =======
 
     - ability to run on more patterns
     - fixes on more runs
+    - run all modules on ansible
 
     Info
     ====
+
     For more information you can consult functions or actual Ansible documentation.
     More information can be also used on http://www.pripravto.cz. You can also
     contact us there.
@@ -277,11 +289,34 @@ class Ansiblator(object):
     def get_host_var(self, host):
         return self.inventory.get_variables(host)
 
-    def parse_inventory(self, inventory):
+    def get_all_modules(self):
+        """list all modules and attach them to Ansbilator"""
+        # constant and code copied from ansible
+        # https://github.com/ansible/ansible/blob/devel/bin/ansible-doc
+        BLACKLIST_EXTS = ('.swp', '.bak', '~', '.rpm')
+        paths = (p for p in module_finder._get_paths() if os.path.isdir(p))
+        modules = []
+        for path in paths:
+            modules.extend(m for m in os.listdir(path) if m not in BLACKLIST_EXTS)
+        for module_name in modules:
+            self.attach_module(module_name)
+
+    def attach_module(self, module_name):
+        """Attach module to Ansiblator"""
+        if hasattr(self, module_name):
+            log.warn('Cannot attach module {}'.format(module_name))
+            return None
+        func = functools.partial(self.run, cmd=None, module_name=module_name, use_ansible=True)
+        setattr(self, module_name, func)
+
+    def parse_inventory(self, inventory, new_filename="ansible_dict_convert"):
         """
         :param inventory: some inventory specification
         :type inventory: dict or str
+        :param new_filename: if you pass dict to inverntory, you can name it
+        :type new_filename: str
         """
+        log.info("Parsing inventory object {}".format(inventory))
         home = os.environ.get('HOME', '/tmp/')
         if isinstance(inventory, str):
             if not inventory.startswith("/"):
@@ -289,7 +324,7 @@ class Ansiblator(object):
             else:
                 filename = "{}".format(inventory)
         elif isinstance(inventory, dict):
-            filename = "{}/{}".format(home, "ansible_hosts")
+            filename = "{}/{}".format(home, new_filename)
             DictToInventory(inventory, filename=filename)
         else:
             pass
@@ -309,7 +344,9 @@ class Ansiblator(object):
                 break
         return data
 
-    def get_host_vars(self, data, su=None, sudo=None, su_pass=None, sudo_pass=None, su_user=None):
+    def get_host_vars(self, data, su=None, sudo=None, su_pass=None,
+                      sudo_pass=None, su_user=None):
+        """get data values from data"""
         if data:
             if su:
                 su_pass = data.get("ansible_su_pass", su_pass)
@@ -370,16 +407,14 @@ class Ansiblator(object):
                 host_list = None
                 info = dict(locals())
                 del info['self']
-                runner = self._runner_ans(**info)
-                self._run(runner, now)
+                self._run(self._runner_ans(**info), now)
         else:
             host_list = [host]
             pattern = None
             data = self.get_host_var(host)
             info = dict(locals())
             del info['self']
-            runner = self._runner_ans(**info)
-            self._run(runner, now)
+            self._run(self._runner_ans(**info), now)
 
     def _runner_ans(self, data=None, cmd=None, pattern=None, module_name=None, inventory=None,
                    now=None, su=None, sudo=None, su_user=None, su_pass=None,
@@ -389,13 +424,14 @@ class Ansiblator(object):
         del info['data']  # remove to skip output of password out to shell
         del info['su_pass']
         del info['sudo_pass']
-        del info['pattern']
+        del info['pattern']  # running pat instead of pattern - FOR CYCLE UP
         log.debug(info)
         cmd = self.create_argument(cmd, kwargs.get('kwargs', {}))
         log.info("Creating new command '{}' on '{}'".format(cmd, module_name))
-        runner = ansible.runner.Runner(pattern=pat, host_list=host_list, module_name=module_name,
+        runner = ansible.runner.Runner(pattern=pat, host_list=host_list,
+                                       module_name=module_name, su_pass=su_pass,
                                        module_args=cmd, inventory=inventory,
-                                       su=su, sudo=sudo, su_user=su_user, su_pass=su_pass,
+                                       su=su, sudo=sudo, su_user=su_user,
                                        sudo_pass=sudo_pass)
         return runner
 
@@ -458,10 +494,12 @@ class Ansiblator(object):
             failed = False
         else:
             failed = True
-        return {'contacted':{'local':{'stdout':out, 'failed':failed , 'stderr':err, 'ret_code':return_c}}, 'dark':{}}
+        return {'contacted':{'local':{'stdout':out, 'failed':failed ,
+                                      'stderr':err, 'ret_code':return_c}}, 'dark':{}}
 
 
-    def runner(self, cmd="uptime", use_ansible=True, module_name="command", inventory=None, now=None, pattern=None, **kwargs):
+    def runner(self, cmd="uptime", use_ansible=True, module_name="command",
+               inventory=None, now=None, pattern=None, **kwargs):
         """
         :param cmd: command to run
         :type cmd: str
@@ -480,22 +518,29 @@ class Ansiblator(object):
         """
         log.info("Making new runner: {}".format(cmd))
         if use_ansible:
-            return self.runner_ans(cmd=cmd, module_name=module_name, inventory=inventory, now=now, pattern=pattern, **kwargs)
+            return self.runner_ans(cmd=cmd, module_name=module_name,
+                                   inventory=inventory, now=now, pattern=pattern,
+                                   **kwargs)
         else:
-            return self.runner_local(cmd=cmd, module_name=module_name, inventory=inventory, now=now, pattern=pattern, ** kwargs)
+            return self.runner_local(cmd=cmd, module_name=module_name,
+                                     inventory=inventory, now=now, pattern=pattern,
+                                     ** kwargs)
 
     run = runner
 
     def copy(self, cmd=None, module_name="copy", src="", dest="", **kwargs):
-        return self.runner(cmd=cmd, use_ansible=True, module_name=module_name, src=src, dest=dest, **kwargs)
+        return self.runner(cmd=cmd, use_ansible=True, module_name=module_name,
+                           src=src, dest=dest, **kwargs)
     put = copy
 
     def fetch(self, cmd=None, module_name="fetch", src="", dest="", **kwargs):
-        return self.runner(cmd=cmd, use_ansible=True, module_name=module_name, src=src, dest=dest, **kwargs)
+        return self.runner(cmd=cmd, use_ansible=True, module_name=module_name,
+                           src=src, dest=dest, **kwargs)
     get = fetch
 
     def ping(self, cmd=None, module_name="ping", **kwargs):
-        return self.runner(cmd=cmd, use_ansible=True, module_name=module_name, **kwargs)
+        return self.runner(cmd=cmd, use_ansible=True, module_name=module_name,
+                           **kwargs)
 
     def local(self, cmd=None, **kwargs):
         return self.runner(cmd=cmd, use_ansible=False, **kwargs)
@@ -510,8 +555,8 @@ class Ansiblator(object):
         """quick check if stop_on_error is active"""
         if self.stop_on_error:
             for k, v in code['contacted'].items():
-                # log.info(v)
-                if v.get('failed', False):  # assuming, that run was mostly succesfull
+                # log.info(v) # assuming, that run was mostly succesfull
+                if v.get('failed', False):
                     raise Exception("ERROR: run was not successfull {}".format(v))
 
     def parse_ret_code(self, code):
@@ -527,17 +572,24 @@ class Ansiblator(object):
 __description__ = Ansiblator.__doc__
 
 if __name__ == "__main__":
-    ans = Ansiblator(pattern="pc", stop_on_error=False)
-    ans.local(["uname", "-a"], now=True)
-    ans.local("uname -a", now=True)
-    ret = ans.local("uname -a", now=True, use_shell=True)
-    if ret['contacted']:
-        for k, v in ret['contacted'].items():
-            print(k, v.get('stdout', None), v.get("stderr", None))
-    ans.run("uptime")
-    ans.run("touch /tmp/aaa.csv")
-    ans.local(["uname", "-a"])
-    ans.get(src="/tmp/aaa.csv", dest="/tmp/")
-    ans.run_all()
+    ans = Ansiblator(pattern="test", stop_on_error=False, run_at_once=True)
+    ans.get_all_modules()
+    ans.copy(src="/tmp/aaa.csv", dest="/tmp/aaa/")
+    ans.user(name="ds")
+    def second_test(ans):
+        ans.local(["uname", "-a"], now=True)
+        ans.local("uname -a", now=True)
+        ret = ans.local("uname -a", now=True, use_shell=True)
+        if ret['contacted']:
+            for k, v in ret['contacted'].items():
+                print(k, v.get('stdout', None), v.get("stderr", None))
+        ans.run("uptime")
+        ans.run_at_once = False
+        ans.run("touch /tmp/aaa.csv")
+        ans.local(["uname", "-a"])
+        ans.get(src="/tmp/aaa.csv", dest="/tmp/")
+        ans.run_all()
+
+    second_test(ans)
 
 
